#!/usr/bin/python

#Cacti Threshold checker plugin for nagios
#jarek@nbcs.rutgers.edu

import MySQLdb
import os,sys
from optparse import OptionParser
import ConfigParser
import signal

#constants
NAGIOS_CODES = { "OK":0, "WARNING":1, "CRITICAL":2, "UNKNOWN":3}
#config file to use if none is specified in the arguments
DEFAULT_CONFIG_FILE = "/usr/lib64/nagios/plugins/rutgers/etc/threshold.cfg"

#number of thresholds triggered
numBadReported=0
#number of thresholds not triggered
numGoodReported=0

#send threshold states to nagios
def submitToNagios(hostname, serviceDesc, output, state):
	# the command we're going to run that sends the data to nagios
	command = NSCA_BIN + " \'" + hostname + "\' \'" + serviceDesc + "\' " + str(state) + " \'" + output + "\' &>/dev/null"
	
	#increment number we've reported to nagios
	global numBadReported,numGoodReported, verboseMode
	if state==0:
		numGoodReported += 1
	else:
		numBadReported += 1
	#run the command, if it fails return a bad state and complain
	if os.system(command) != 0:
		print "CRITICAL: error running " + command
		sys.exit(NAGIOS_CODES['CRITICAL'])
	if verboseMode:
		print command	
		
#get hosts in threshold DB that went over their thresholds, and call submitToNagios w/ them
#all parameters are database connection information
def checkDB(DBhost,DBuser,DBpasswd,DBname):
	try:
		#connect to the DB 
		conn = MySQLdb.connect (host=DBhost, user=DBuser,passwd=DBpasswd,db=DBname)
		cursor = conn.cursor()
		#get info from all rows
		query = "SELECT name,host_id,thold_type,thold_hi,thold_low,lastread,thold_alert,bl_alert,bl_enabled FROM thold_data"
		cursor.execute(query)
		#for each row, get the hostname from a different talbeu, and report its status to nagios
		for row in cursor.fetchall():

			#grab units out of the threshold name
			#they are encased in [] and MUST be the last thing in the threshold name
			# (anything after them gets cut off)
			thresholdName = row[0]
			leftIndex = thresholdName.find("[")
			rightIndex = thresholdName.find("]")
			if leftIndex == -1 or rightIndex == -1:
				print "CRITICAL: Threshold '" + thresholdName + "' does not contain units surrounded by []";
				sys.exit (NAGIOS_CODES['CRITICAL'])
			units = thresholdName[leftIndex:rightIndex+1];
			thresholdName = thresholdName[:leftIndex-1]

			# grab the hostname to use for nagios from the device notes field on cacti
			# if it doesn't exist, bad things happen
			cursor.execute("SELECT notes FROM host WHERE id=" + str(row[1]))
			hostNotes = cursor.fetchone();
			hostname = hostNotes[0]
			hostname = hostname.replace("NagiosHostname:","")

			#is it a hard hi/lo value?
			if row[2] == 0:
				#is it triggered?
				if (row[6] !=0):
					status = 2
					stateString = "CRITICAL"
				else:
					status = 0
					stateString = "OK"
				if not row[4]:
					lowThreshold ='None'
				else:
					lowThreshold = str(row[4])
				if not row[3]:
					highThreshold='None'
				else:
					highThreshold=str(row[3])

				desc_string = stateString + " " +   str(row[5]) + " " +units +  " Thresholds (" + lowThreshold + "," + highThreshold + ")" 
			#is it a baseline?
			elif row[8] == "on":
				#is it triggered low?
				if row[7] == 1:
					desc_string = "baseline low (" + str(row[5]) + ")"
					status =2
				#is it triggered high?
				elif row[7] == 2:
					desc_string = "baseline high (" + str(row[5]) + ")"
					status =2
				#it must be ok
				else:
					desc_string = "baseline ok (" + str(row[5]) + ")"
					status =0
			else:
				desc_string = "Not a hard high/low value or a baseline"
				status = 3

			submitToNagios(hostname,thresholdName,desc_string,status)
	#if something went wrong, exit with an error and complain
	except MySQLdb.Error, e:
		print "CRITICAL: Database Error %d: %s" % (e.args[0], e.args[1])
		sys.exit (NAGIOS_CODES['CRITICAL'])
#called if the plugin is taking too long to execute
def timedOut():
	print "CRITICAL: timed out"
	sys.exit( NAGIOS_CODES['CRITICAL'])
	

#begin program execution

#get CLI arguments
parser = OptionParser()
parser.add_option("-f","--file",action="store",type="string",dest="config_file", help="Configuration file to use")
parser.add_option("-v","--verbose",action="store_true",default=False,dest="verboseMode", help="Enable verbose output")
(options,args) = parser.parse_args()
if not options.config_file:
	config_file = DEFAULT_CONFIG_FILE
else:
	config_file = options.config_file
verboseMode=options.verboseMode

#read settings from config file
try:
	config = ConfigParser.ConfigParser()
	config.read(config_file)
	databaseHost = config.get("main","DatabaseHost")
	databaseUser = config.get("main","DatabaseUser")
	databasePasswd = config.get("main","DatabasePasswd")
	databaseName = config.get("main","DatabaseName")
	NSCA_BIN = config.get("main","NSCACommand")
	timeoutSeconds = int( config.get("main","TimeoutSeconds") )
except ConfigParser.NoSectionError,e:
	print "CRITICAL: error reading from config file \'" + config_file +"\' Either it isn't readable, or is missing the section \'main\'"
	sys.exit( NAGIOS_CODES['CRITICAL'])
except ConfigParser.NoOptionError, e:
	print "CRITICAL: error reading from config file \'" + config_file +"\' Missing value for setting \'"+ e.option+"\' in section \'"+ e.section + "\'"
	sys.exit( NAGIOS_CODES['CRITICAL'])

#if the plugin takes longer than timeoutSeconds to run, it times out and we call timedOut()
signal.signal(signal.SIGALRM,timedOut)
signal.alarm(timeoutSeconds)

#check the database, and report all threshold statuses to nagios
checkDB(databaseHost,databaseUser,databasePasswd,databaseName)

#print out status, and return OK (if we got this far without an error, status must be OK)
print "OK:",numBadReported,"threshold(s) triggered,", numGoodReported, "not triggered"
sys.exit(NAGIOS_CODES['OK'])
